@@ -5,6 +5,7 @@ import sys
 from contextlib import contextmanager
 from datetime import datetime
 from pathlib import Path
+from shutil import rmtree
 from subprocess import DEVNULL, PIPE, CalledProcessError
 from subprocess import Popen as SPOpen
 from subprocess import run as srun
@@ -153,7 +154,7 @@ def get_tags():
 
 def darcs_clone(source, destination):
     """Clone git-repo."""
-    run(["darcs", "clone", "--no-working-dir", source, destination], check=True)
+    run(["darcs", "clone", source, destination], check=True)
 
 
 def git_try_fast_forward(rev, last):
@@ -247,7 +248,7 @@ def record_all(rev, *, last=None, postfix=None, comments=None):
     by = author(rev)
     if postfix:
         msg = f"{msg} {postfix}"
-    if comments:
+    elif comments:
         msg = f"{msg}\n\n{comments}"
     with _darcs_comment.open("w", encoding="UTF-8") as f:
         f.write(msg)
@@ -435,24 +436,19 @@ def transfer(gen, count, *, last=None):
                 if git_try_fast_forward(rev, last):
                     record_revision(rev, last=last)
                     last = rev
-                    recorded = True
                     records += 1
                     if records % 100 == 0:
                         checkpoint(last)
                 pbar.update()
                 if _shutdown:
                     sys.exit(0)
-            if not recorded:
-                record_revision(rev, last=last)
-                last = rev
     except Exception:
         print(f"Failed on revision {last}")
         raise
-    finally:
-        checkpoint(last)
+    return last
 
 
-def import_range(rbase):
+def import_range(rbase, *, from_checkpoint=False):
     """Run the transfer to darcs."""
     rhead = get_head()
     if rbase == rhead:
@@ -466,8 +462,16 @@ def import_range(rbase):
     wipe()
     checkout(rbase)
     with less_boring():
-        record_all(rbase)
-        transfer(gen, count, last=rbase)
+        try:
+            last = rbase
+            if not from_checkpoint:
+                record_all(rbase)
+            last = transfer(gen, count, last=last)
+            if last != rhead:
+                checkout(rhead)
+                record_all(rhead)
+        finally:
+            checkpoint(rhead)
 
 
 def import_one():
@@ -522,7 +526,7 @@ def clone(source, destination, verbose):
     repo_source = Path(darcs_dest, "_darcs")
     repo_dest = Path(destination, "_darcs")
     repo_source.rename(repo_dest)
-    darcs_dest.rmdir()
+    rmtree(darcs_dest, ignore_errors=True)
     os.chdir(destination)
     relink()
 
@@ -559,7 +563,9 @@ def update(verbose, base, warn, shallow):
     if not Path("_darcs").exists():
         initialize()
     rbase = get_lastest_rev()
+    from_checkpoint = False
     if rbase:
+        from_checkpoint = True
         do_one = False
         if base:
             print("Found git-checkpoint, ignoring base-option")
@@ -588,7 +594,7 @@ def update(verbose, base, warn, shallow):
         if do_one:
             import_one()
         else:
-            import_range(rbase)
+            import_range(rbase, from_checkpoint=from_checkpoint)
         failed = False
     finally:
         if branch:
